//  Display
// 
//  Version	Date	 Author	 Description
//	--------------------------------------------------------------------------
//  0.01	24/06/01 BWW	 Initial implementation

#include <c:\psx\include\libps.h>
#include "planner.h"
#include "funcprototypes.h"

extern	Agent_Struct			Agent;
extern	Key_Struct				Key;
extern	Start_Struct			Start;
extern	Exit_Struct				Exit;
extern	Door_Struct				Door[];
extern	Bridge_Struct			Bridge[];
extern	Route_Struct	 		Route;
extern	SearchGoalNodes_Struct	SearchGoalNodes;
extern	Plan_Struct				Plan;
extern	ActionEffects_Struct	ActionEffects;

// Standard Text
extern	char 	TurnMsg[];
extern	char 	PlanMsg[];
extern	char 	StepMsg[];
extern	char	NoPlanMsg[];
extern	char 	FailedPlanMsg[];
extern	long	Turn;
extern	char 	RouteMsg[];
extern	char 	GoalMsg[];
extern	char 	NotGoalMsg[];

extern	char 	NoActionMsg[];
extern	char 	MoveToMsg[];
extern	char 	UnlockMsg[];
extern	char 	OpenMsg[];
extern	char 	PickupMsg[];
extern	char 	WaitMsg[];
extern	char 	PlanActionMsg[];

extern	char 	ComplexityLowMsg[];
extern	char 	ComplexityMediumMsg[];
extern	char 	ComplexityHighMsg[];

extern	char 	SpeedLowMsg[];
extern	char 	SpeedMediumMsg[];
extern	char 	SpeedHighMsg[];

extern	char	StartLocationMsg[];

extern	int		EnvSpeed;
extern	int		EnvComplexity;
extern	int		EnvStartLocation;

extern	char 	Environment[ENV_HEIGHT][ENV_WIDTH];

extern	int		BRIDGENUM;
extern	int		DOORNUM;


// Flag whether to print Debug information
extern	int		Debug;

// Sprite structure
extern	GsSPRITE 	MessageSprite;

// Ordering table variables
extern	GsOT		WorldOT[2];
extern	GsOT_TAG	OTTags[2][1<<OT_LENGTH];

// Which buffer variable
extern	int			activeBuff;


//	--------------------------------------------------------------------------
// 	Display standard text
//	--------------------------------------------------------------------------
//	Description:	Display standard text on the screen
//
//	Parameters:		None
//	Returns:		None

void DisplayStandardText()
	{

//	Update and display the current turn
	Turn++;
	NumberDisplay(Turn,70,162,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH);
	MessageDisplay(TurnMsg,16,162,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,4);

//  Display the number of goal routes returned from the last search
	NumberDisplay(SearchGoalNodes.numgoalnodes,92,176,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH);
	MessageDisplay(RouteMsg,16,176,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,6);
	
//  Display the duration remaining on the current step
	NumberDisplay(Plan.duration,70,190,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH);
	MessageDisplay(StepMsg,16,190,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,4);
	
	// Plan status	
	switch (Plan.status)
		{
		case PLAN:
			MessageDisplay(PlanMsg,148,162,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,7);
			break;
		case NOPLAN:
			MessageDisplay(NoPlanMsg,148,162,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,7);
			break;
		case FAILEDPLAN:
			MessageDisplay(FailedPlanMsg,148,162,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,11);
			break;
		}
	
	// The current action
	if (Plan.actioneffectindex == -1)
		{
		MessageDisplay(NoActionMsg,148,176,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,9);
		}
	else
		{
		switch (ActionEffects.effect[Plan.actioneffectindex].condition)
			{
			case EFFECT_MOVETO:
				MessageDisplay(MoveToMsg,148,176,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,6);
				break;
			case EFFECT_UNLOCK:
				MessageDisplay(UnlockMsg,148,176,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,6);
				break;
			case EFFECT_OPEN:
				MessageDisplay(OpenMsg,148,176,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,4);
				break;
			case EFFECT_PICKUP:
				MessageDisplay(PickupMsg,148,176,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,6);
				break;
			case EFFECT_PLAN:
				MessageDisplay(PlanActionMsg,148,176,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,4);
				break;
			case EFFECT_WAIT:
				MessageDisplay(WaitMsg,148,176,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,4);
				break;
			}
		}

	// Whether the agent's goal has been achieved
	if (Plan.goalachieved == false)
		{
		MessageDisplay(NotGoalMsg,148,190,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,8);
		}
	else
		{
		MessageDisplay(GoalMsg,148,190,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,12);
		}

	// Speed and complexity of the dynamic environment	
	switch (EnvSpeed)
		{
		case SPEED_LOW:
			MessageDisplay(SpeedLowMsg,16,204,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,9);
			break;
		case SPEED_MEDIUM:
			MessageDisplay(SpeedMediumMsg,16,204,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,9);
			break;
		case SPEED_HIGH:
			MessageDisplay(SpeedHighMsg,16,204,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,10);
			break;
		}
	switch (EnvComplexity)
		{
		case COMPLEXITY_LOW:
			MessageDisplay(ComplexityLowMsg,148,204,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,11);
			break;
		case COMPLEXITY_MEDIUM:
			MessageDisplay(ComplexityMediumMsg,148,204,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,11);
			break;
		case COMPLEXITY_HIGH:
			MessageDisplay(ComplexityHighMsg,148,204,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,12);
			break;
		}

//  Display the agent start location
	NumberDisplay(EnvStartLocation,80,222,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH);
	MessageDisplay(StartLocationMsg,16,222,TXT_SIZE_MEDIUM,TXT_SPACE_MEDIUM, ZPRIORITY_HIGH,5);

	}

//	--------------------------------------------------------------------------
// 	Display Physical Route
//	--------------------------------------------------------------------------
//	Description:	Display the physical route that the agent is currently 
//					following.
//
//	Parameters:		None
//	Returns:		None

void DisplayRoute()
	{
	char	chartoprint[2];
	int		index;
	
	chartoprint[0] = OBJ_ROUTE;
	chartoprint[1] = '\n';

	// Display all of the steps on the route
	for (index=0;index<Route.numsteps;index++)
		{
		MessageDisplay(chartoprint,(ENV_X+(Route.x[index]*ENV_SPRITESIZE)),(ENV_Y+(Route.y[index]*ENV_SPRITESIZE)),ONE,ENV_SPRITESIZE, ZPRIORITY_HIGH,1);
		}	

	}

//	--------------------------------------------------------------------------
// 	Display the Environment on Screen 
//	--------------------------------------------------------------------------
//	Description:	Display a graphic representation of the environment on screen
//
//	Parameters:		None
//	Returns:		None

void DisplayEnvironment()
	{
	char	chartoprint[2];
	int		index, index2;

//	printf("Display environment routine\n");

	for (index=0;index < ENV_HEIGHT;index++)
		{
		for (index2=0;index2 < ENV_WIDTH;index2++)
//		for (index2=0;index2 < 1;index2++)
			{
			chartoprint[0] = Environment[index][index2];
//			chartoprint[1] = 'n';
			MessageDisplay(chartoprint,(ENV_X+(index2*ENV_SPRITESIZE)),(ENV_Y+(index*ENV_SPRITESIZE)),
							ONE,ENV_SPRITESIZE, ZPRIORITY_MED,1);
			}
		}
	}

//	--------------------------------------------------------------------------
// 	Display objects
//	--------------------------------------------------------------------------
//	Description:	Display the objects that exist within the environment
//
//	Parameters:		None
//	Returns:		None

void DisplayObject()
	{
	char	chartoprint[2];
	int		index;

	// Agent
	chartoprint[0] = OBJ_AGENT;
	chartoprint[1] = '\n';
	MessageDisplay(chartoprint,(ENV_X+(Agent.x*ENV_SPRITESIZE)),(ENV_Y+(Agent.y*ENV_SPRITESIZE)),ONE,ENV_SPRITESIZE, ZPRIORITY_HIGH,1);

	// Key
	chartoprint[0] = OBJ_KEY;
	chartoprint[1] = '\n';
	MessageDisplay(chartoprint,(ENV_X+(Key.x*ENV_SPRITESIZE)),(ENV_Y+(Key.y*ENV_SPRITESIZE)),ONE,ENV_SPRITESIZE, ZPRIORITY_HIGH,1);

	// Start
	chartoprint[0] = OBJ_START;
	chartoprint[1] = '\n';
	MessageDisplay(chartoprint,(ENV_X+(Start.x*ENV_SPRITESIZE)),(ENV_Y+(Start.y*ENV_SPRITESIZE)),ONE,ENV_SPRITESIZE, ZPRIORITY_HIGH,1);

	// Exit
	chartoprint[0] = OBJ_EXIT;
	chartoprint[1] = '\n';
	MessageDisplay(chartoprint,(ENV_X+(Exit.x*ENV_SPRITESIZE)),(ENV_Y+(Exit.y*ENV_SPRITESIZE)),ONE,ENV_SPRITESIZE, ZPRIORITY_HIGH,1);
	
	// Doors
	for (index=0;index<DOORNUM;index++)
		{
		chartoprint[0] = OBJ_DOOR;
		chartoprint[1] = '\n';
		MessageDisplay(chartoprint,(ENV_X+(Door[index].x*ENV_SPRITESIZE)),(ENV_Y+(Door[index].y*ENV_SPRITESIZE)),ONE,ENV_SPRITESIZE, ZPRIORITY_MED,1);
		}
		
	// Bridges
	for (index=0;index<BRIDGENUM;index++)
		{
		chartoprint[0] = OBJ_BRIDGE;
		chartoprint[1] = '\n';
		MessageDisplay(chartoprint,(ENV_X+(Bridge[index].x*ENV_SPRITESIZE)),(ENV_Y+(Bridge[index].y*ENV_SPRITESIZE)),ONE,ENV_SPRITESIZE, ZPRIORITY_MED,1);
		}
	}


//	--------------------------------------------------------------------------
// 	Display a Text Message 
//	--------------------------------------------------------------------------
//	Description:	Display a text string on screen
//
//	Parameters:		Char[]	Message string to display
//					Int		X Position
//					Int 	Y Position
//					Short	Size scaling factor - 4096 = full size
//					Int		Physical screen offset between each character
//					Short	On screem Z priority
//	Returns:		None

void MessageDisplay(char DisplayMessage[], int XPos, int YPos, short Scale, int Offset, short ZPriority, int length)
	{
	// Local variables
	int index;
	int character, lcharacter;
	int spriteu, spritev;
	
//	int length;

	if (Debug == true)
		{
		printf ("Index value: %i \n", strlen(DisplayMessage));
		}
		
		
//	length=(strlen(DisplayMessage) - 1);
	
	// For each character in the string
//	for (index = 0; index < (strlen(DisplayMessage) - 1); index++)
	for (index = 0; index < length; index++)
		{
		
		// Get the character as an Int
		character = (int)(DisplayMessage[index]);
		
		// Check for a space
		if (character == 32) 
			{
			spriteu = 32;
			spritev = 48;
			}
		
		// Check for graphic characters
		if ((character > 96) && (character < 112))
			{
			// Calculate the offset (x & y) within the sprite for the required character
			lcharacter = character - 97;
			spriteu = (lcharacter - ((lcharacter / 8)*8)) * 16;
			spritev = ((lcharacter / 8) * 16) + 96;
			}

		// Check for A to Z
		if ((character > 64) && (character < 91))
			{
			// Calculate the offset (x & y) within the sprite for the required character
			lcharacter = character - 65;
			spriteu = (lcharacter - ((lcharacter / 8)*8)) * 16;
			spritev = (lcharacter / 8) * 16;
			}

		// Check for 0 to 9
		if ((character > 47) && (character < 58))
			{
			// Calculate the offset (x & y) within the sprite for the required character
			lcharacter = character - 48;
			spriteu = (lcharacter - ((lcharacter / 8)*8)) * 16;
			spritev = ((lcharacter / 8) * 16) + 64;
			}
			
		// Add the character to the OT
		MessageSprite.x			= XPos;
		MessageSprite.y			= YPos;
		MessageSprite.u			= spriteu;
		MessageSprite.v			= spritev;
		MessageSprite.scalex	= Scale;
		MessageSprite.scaley	= Scale;
		GsSortSprite(&MessageSprite,&WorldOT[activeBuff],ZPriority);
		
		// Move the display position (only a half move for 'i','1' & space
		if ((character == 32)||(character == 73)||(character == 49))
			{
			XPos += (Offset/2);
			}
		else
			{
			XPos += Offset;
			}
		}
	}

//	--------------------------------------------------------------------------
// 	Display a Number 
//	--------------------------------------------------------------------------
//	Description:	Display a number on screen
//
//	Parameters:		long	Number to display
//					Int		X Position
//					Int 	Y Position
//					Short	Size scaling factor - 4096 = full size
//					Int		Physical screen offset between each character
//					Short	On screem Z priority
//	Returns:		None

void NumberDisplay(long Num, int XPos, int YPos, short Scale, int Offset, short ZPriority)
	{
	// Local variables
	int 	index, character, lcharacter, spriteu, spritev,	digit;
	char	string[11];
	
	strcpy(string, "");
		
	while (Num > 0)
		{
		digit = (Num - ((Num / 10) * 10));
		if (digit == 0)
			{
			strcat(string, "0");
			}
		if (digit == 1)
			{
			strcat(string, "1");
			}
		if (digit == 2)
			{
			strcat(string, "2");
			}
		if (digit == 3)
			{
			strcat(string, "3");
			}
		if (digit == 4)
			{
			strcat(string, "4");
			}
		if (digit == 5)
			{
			strcat(string, "5");
			}
		if (digit == 6)
			{
			strcat(string, "6");
			}
		if (digit == 7)
			{
			strcat(string, "7");
			}
		if (digit == 8)
			{
			strcat(string, "8");
			}
		if (digit == 9)
			{
			strcat(string, "9");
			}
		Num = Num / 10;
		}
	strcat(string, "\n");
	
//	printf(string);
//	printf("Length of string = %d \n",(strlen(string)-1));

	// For each character in the string
	for (index = (strlen(string)-2); index > -1; index--)
		{
		
		// Get the character as an Int
		character = (int)(string[index]);
		
		// Calculate the offset (x & y) within the sprite for the required character
		lcharacter = character - 48;
		spriteu = (lcharacter - ((lcharacter / 8)*8)) * 16;
		spritev = ((lcharacter / 8) * 16) + 64;
//		printf("A number ..\n");
			
		// Add the character to the OT
		MessageSprite.x			= XPos;
		MessageSprite.y			= YPos;
		MessageSprite.u			= spriteu;
		MessageSprite.v			= spritev;
		MessageSprite.scalex	= Scale;
		MessageSprite.scaley	= Scale;
		GsSortSprite(&MessageSprite,&WorldOT[activeBuff],ZPriority);
		
		// Move the display position (only a half move for 'i','1' & space
		if (character == 49)
			{
			XPos += (Offset/2);
			}
		else
			{
			XPos += Offset;
			}
		}
	}

//	--------------------------------------------------------------------------
// 	Indicate whether Bridges and Doors are accessible
//	--------------------------------------------------------------------------
//	Description:	Display a tick or cross on screen to indicate whether each
//					Bridge or Door is accessible
//
//	Parameters:		None
//	Returns:		None

void DisplayAccessibility()
	{
	char	chartoprint[2];

	int		index;

	// All of the doors
	for (index=0;index<DOORNUM;index++)
		{
		if ((Door[index].doorstatus == DOOROPEN)&&(Door[index].lockstatus == DOORUNLOCKED))
			{	
			chartoprint[0] = OBJ_TICK;
			}
		else
			{
			chartoprint[0] = OBJ_CROSS;
			}
		chartoprint[1] = '\n';
		MessageDisplay(chartoprint,(ENV_X+(Door[index].x*ENV_SPRITESIZE)),(ENV_Y+(Door[index].y*ENV_SPRITESIZE)),ONE,ENV_SPRITESIZE, ZPRIORITY_HIGH,1);
		}	

	// All of the bridges
	for (index=0;index<BRIDGENUM;index++)
		{
		if (Bridge[index].status == BRIDGELOWERED)
			{	
			chartoprint[0] = OBJ_TICK;
			}
		else
			{
			chartoprint[0] = OBJ_CROSS;
			}
		chartoprint[1] = '\n';
		MessageDisplay(chartoprint,(ENV_X+(Bridge[index].x*ENV_SPRITESIZE)),(ENV_Y+(Bridge[index].y*ENV_SPRITESIZE)),ONE,ENV_SPRITESIZE, ZPRIORITY_HIGH,1);
		}	

	}


