// Metrics.c
//
// Version	Date	 Author	 Description
//	--------------------------------------------------------------------------
//  0.01	04/08/01 BWW	 Initial implementation

#include <c:\psx\include\libps.h>
#include "planner.h"
#include "funcprototypes.h"

extern	char 	Environment[ENV_HEIGHT][ENV_WIDTH];


extern	Metrics_Struct	Metrics;

extern	int		EnvSpeed;
extern	int		EnvComplexity;
extern	int		EnvStartLocation;

extern	int	BRIDGENUM;
extern	int	DOORNUM;

//	--------------------------------------------------------------------------
// 	Output Complexity Metrics
//	--------------------------------------------------------------------------
//	Description:	Output metrics for the complexity of the current scenario
//
//	Parameters:		None
//	Returns:		None

void MetricComplexity()
	{
	
	int index, index2;
	
	// Ouput the header for the columns
	printf("Scenario Complexity,Rate of Change,Environment Size,Accessible Locations,Inaccessible Locations,Dynamic Objects,Other Objects,Total Objects\n");
	
	// Calculate the values to output
	Metrics.c_size = (ENV_WIDTH * ENV_HEIGHT);
	
	for (index=0;index<ENV_WIDTH;index++)
		{
		for (index2=0;index2<ENV_HEIGHT;index2++)
			{
			if ((Environment[index][index2]==OBJ_WATER)||(Environment[index][index2]==OBJ_WALL))
				{
				Metrics.c_inaccessible++;
				}
			else
				{
				Metrics.c_accessible++;
				}
			}
		}
	
	Metrics.c_dynamic_objects = BRIDGENUM;
	Metrics.c_other_objects = (DOORNUM+1);
	Metrics.c_total_objects = (Metrics.c_dynamic_objects + Metrics.c_other_objects);

	Metrics.c_change = EnvSpeed;
	Metrics.c_complexity = EnvComplexity;
	
	printf("%d,%d,%d,%d,%d,%d,%d,%d\n", Metrics.c_complexity, Metrics.c_change, Metrics.c_size, 
			Metrics.c_accessible, Metrics.c_inaccessible,Metrics.c_dynamic_objects, 
			Metrics.c_other_objects, Metrics.c_total_objects);
	
	}
	


//	--------------------------------------------------------------------------
// 	Output metrics for an agent plan
//	--------------------------------------------------------------------------
//	Description:	Output the metrics for the current instance of an agent plan
//
//	Parameters:		None
//	Returns:		None

void MetricPlan()
	{
	
	printf("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",Metrics.c_complexity, Metrics.c_change,EnvStartLocation,
			Metrics.p_plannumber, Metrics.p_numpreconditions,	
			Metrics.p_numactioneffects, Metrics.p_numcausallinks, Metrics.p_numorderings,
			Metrics.e_preconditionsmet, Metrics.e_preconditionsfailed,
			Metrics.e_numsearches, Metrics.e_numsearchnodes, Metrics.e_numgoalnodes);
	
	}
	
//	--------------------------------------------------------------------------
// 	Output agent plan summary information
//	--------------------------------------------------------------------------
//	Description:	Output summary information covering all of the plans that an
//					agent has constructed during the execution of the scenario.
//
//	Parameters:		None
//	Returns:		None

void MetricPlanSummary()
	{
	
	printf("Scenario Complexity,Rate of Change,StartLocation,Total Plan Number,Total Number Precondiditions,Total Number Action Effects,Total Number Causal Links,Total Number Orderings,Total Preconditions Met,Total Preconditions Failed,Total Number Searches,Total Search Nodes,Total Goal Nodes\n");
	printf("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d\n",Metrics.c_complexity, Metrics.c_change,EnvStartLocation,
			Metrics.p_totplannumber, Metrics.p_totnumpreconditions,	
			Metrics.p_totnumactioneffects, Metrics.p_totnumcausallinks, Metrics.p_totnumorderings,
			Metrics.e_preconditionsmet, Metrics.e_preconditionsfailed,
			Metrics.e_numsearches, Metrics.e_numsearchnodes, Metrics.e_numgoalnodes);
	
	}
	
//	--------------------------------------------------------------------------
// 	Output column headers for the agent plan metrics
//	--------------------------------------------------------------------------
//	Description:	Output the headers for the Excel columns into which we'll put
// 					metric info about agent plans
//
//	Parameters:		None
//	Returns:		None

void MetricPlanHeader()
	{
	printf("Scenario Complexity,Rate of Change,Start Location,Plan Number,Number Precondiditions,Number Action Effects,Number Causal Links,Number Orderings,Cumulative Preconditions Met,Cumulative Preconditions Failed,Cumulative Number Searches,Cumulative Search Nodes,Cumulative Goal Nodes\n");
		
	}
	
//	--------------------------------------------------------------------------
// 	Initialise the Metrics variables
//	--------------------------------------------------------------------------
//	Description:	Initialise the variables that we will use to capture metric
// 					information during the execution of the scenario.
//
//	Parameters:		None
//	Returns:		None

void InitMetrics()
	{
	
	Metrics.p_plannumber = 0;
	Metrics.p_numpreconditions = 0;
	Metrics.p_numactioneffects = 0;
	Metrics.p_numcausallinks = 0;
	Metrics.p_numorderings = 0;
	Metrics.p_totplannumber = 0;
	Metrics.p_totnumpreconditions = 0;
	Metrics.p_totnumactioneffects = 0;
	Metrics.p_totnumcausallinks = 0;
	Metrics.p_totnumorderings = 0;
	Metrics.e_preconditionsmet = 0;
	Metrics.e_preconditionsfailed = 0;
	Metrics.e_numsearches = 0;
	Metrics.e_numsearchnodes = 0;
	Metrics.e_numgoalnodes = 0;


	}